#!/usr/bin/env python

from distutils.core import setup

setup(name='openalpr',
      version='1.0',
      description='OpenALPR Python Bindings',
      author='Matt Hill',
      author_email='matthill@openalpr.com',
      url='http://www.openalpr.com/',
      packages=['openalpr']
     )
