/*
 * Copyright (c) 2015 OpenALPR Technology, Inc.
 * Open source Automated License Plate Recognition [http://www.openalpr.com]
 *
 * This file is part of OpenALPR.
 *
 * OpenALPR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "segmentationgroup.h"

namespace alpr
{

  SegmentationGroup::SegmentationGroup()
  {

  }

  SegmentationGroup::~SegmentationGroup()
  {

  }

  void SegmentationGroup::add(int segmentID)
  {
    this->segmentIDs.push_back(segmentID);
  }

  bool SegmentationGroup::equals(SegmentationGroup otherGroup)
  {
    if (segmentIDs.size() != otherGroup.segmentIDs.size())
      return false;

    for (int i = 0; i < segmentIDs.size(); i++)
    {
      if (otherGroup.segmentIDs[i] != segmentIDs[i])
        return false;
    }

    return true;
  }
  
}